package business;

import http.HttpBinaryServlet;
import http.HttpTextServlet;
import http.Server;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

import util.Json;
import business.publishers.Publisher;
import business.publishers.SeLoger;

import com.sun.net.httpserver.HttpExchange;

public class ImmoServer extends Server {
	private Class<?>[] publisherClasses = { SeLoger.class };

	protected void init() {
		server.createContext("/").setHandler(new HttpBinaryServlet() {
			@Override
			protected void doGet(HttpExchange exchange, BufferedOutputStream out) throws IOException {
				String url = exchange.getRequestURI().toString();
				if ("/".equals(url))
					url = "/index.html";

				exchange.getResponseHeaders().add("Content-Type", getContentType(url));
				exchange.sendResponseHeaders(200, 0);

				BufferedInputStream bis = new BufferedInputStream(getClass().getResourceAsStream("/data" + url));
				byte[] buf = new byte[1024];
				int size;
				while ((size = bis.read(buf)) > -1) {
					out.write(buf, 0, size);
				}
			}
		});
		server.createContext("/services/autocomplete").setHandler(new HttpTextServlet() {
			@Override
			protected void doGet(HttpExchange exchange, PrintWriter out) throws IOException {
				exchange.getResponseHeaders().add("Content-Type", "text/javascript");
				exchange.sendResponseHeaders(200, 0);

				String namePrefix = getParameters(exchange).get("sCity");
				out.println(Json.encode(Insee.instance.getCitiesStartingBy(namePrefix, 5)));
			}
		});
		server.createContext("/services/ads").setHandler(new HttpTextServlet() {
			@Override
			protected void doGet(HttpExchange exchange, PrintWriter out) throws IOException {
				exchange.getResponseHeaders().add("Content-Type", "text/html");
				exchange.sendResponseHeaders(200, 0);

				HttpParameters params = new HttpParameters(exchange);

				String[] cityParts = params.get("sCity").split("-");
				City city = Insee.instance.getCity(cityParts[0].trim(), cityParts[1].trim());
				if (city == null) {
					out.println("Ville incorrecte: " + params.get("sCity"));
					out.println("Format attendu : VILLE - CODE POSTAL");
				} else {
					Kind kind = params.getEnum(Kind.class, "sKind");
					int minPrice = params.getInt("iMinPrice");
					int maxPrice = params.getInt("iMaxPrice");
					int minArea = params.getInt("iMinArea");
					int maxArea = params.getInt("iMaxArea");

					try {
						for (Class<?> c : publisherClasses) {
							Publisher p = (Publisher) c.newInstance();
							List<Ad> ads = p.findAds(kind, city, minPrice, maxPrice, minArea, maxArea);
							for (Ad ad : ads) {
								out.println(ad.getHtml());
							}
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		});
	}

	public static void main(String[] args) throws IOException {
		ImmoServer server = new ImmoServer();
		server.start();
		System.out.println("server started");
		System.in.read();
		server.stop();
	}
}
