package business.publishers;

import http.Client;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import util.Constants;
import util.StringHelper;
import business.Ad;
import business.City;
import business.Kind;

public class SeLoger extends Publisher{
	private static Pattern cleanupRe;
	private static Pattern adsRe;
	private static Pattern nbPagesRe;
    private static String findAdsQueryTemplate = "http://www.seloger.com/recherche.htm?euro=1&idtt=2&idtypebien={typeBien}&pxmin={pxMin}&pxmax={pxMax}&surfacemin={surfMin}&surfacemax={surfMax}&ci={ci}&immobw_ann_qrypg={page}";

	static{
		cleanupRe = Pattern.compile("(<td class=addcredit.*?</td>)" // crédit
				+ "|<span class=addprixfr.*?</span>" // prices in francs
				+ "|(<span class=\"mentionfai\".*?</span>)" // FAI
				+ "|(<div class=\"addmapratp\".*?</div>)" // localisation RATP
				+ "|(onclick=\".*?\")" // evenements javascript
				+ "|(<span><a .*?class=\"addliendetail\">.*?</span>)" // +details
				+ "|(<div class=\"addmapfrance\".*?</div>)" // localisation
				+ "|(<div class=\"addbasv4\".*?</div>)" // financement
				+ "|(<div class=\"boxpubmasterv2\".*?</div>)" // pub CA
				+ "|(<td class=\"addRef\".*?</td>)" // MAJ/REF
				+ "|(<td class=\"addContact\".*?</td>)" // Contact agence
				+ "|(<div class=\"addContactPlus\".*?</div>)" // Contact agence
				+ "|(<script>dispLieux('');</script>)"
		, Constants.REG_EXP_FLAGS);
		adsRe = Pattern.compile("(<table oc=\"add.*?</table>.*?<table.*?</table>.*?</table>)", Constants.REG_EXP_FLAGS);
		nbPagesRe = Pattern.compile("<b>Page (.*?) sur (.*?).</b>", Constants.REG_EXP_FLAGS);
	}
	
	protected List<Ad> findAds(Kind kind, City city, int minPrice, int maxPrice, int minArea, int maxArea, int nbPage) {
		if (nbPage < 1) throw new RuntimeException("nbPage in [1;inf[");
		
		List<Ad> ads = new ArrayList<Ad>();
        
        Object[][] replacements = {
    		{"typeBien", (kind == Kind.Flat ? "1" : "2")},
        	{"{pxMin}", minPrice}, {"{pxMax}", maxPrice},
        	{"{surfaceMin}", minArea}, {"{surfaceMax}", maxArea},
        	// SeLoger Hack
        	{"{ci}", new StringBuilder(city.getInseeCode()).insert(city.getInseeCode().length() - 3, "0")}, 
        	{"{page}", nbPage}
        };
        String findAdsQuery = StringHelper.replace(findAdsQueryTemplate, replacements);
        StringBuilder rawHtml = Client.get(findAdsQuery);        
        String html = cleanup(rawHtml);

        // Set the nb of pages (only when accessing the first page)
        if (nbPage == 1) {
        	Matcher nbPagesMatcher = nbPagesRe.matcher(html);
        	nbPages = nbPagesMatcher.find() ? 
        			Integer.parseInt(nbPagesMatcher.group(2)) : 1;
        }

        // Only extract ads
		Matcher adsMatcher = adsRe.matcher(html);
		while(adsMatcher.find()){
			ads.add(new Ad(adsMatcher.group(1)));
		}

        return ads;
    }

	private String cleanup(StringBuilder rawHtml) {
		String html = cleanupRe.matcher(rawHtml).replaceAll("");
		html = html.replace("<a href=\"/", "<a href=\"http://www.seloger.com/");
		
		// Cleanup images
		Matcher imgMatcher = Pattern.compile("<img .*?>").matcher(html);
		StringBuffer sb = new StringBuffer();
		while (imgMatcher.find()){
			if (! imgMatcher.group(0).contains("thumbnails")){
				imgMatcher.appendReplacement(sb, "");
			}
		}
		imgMatcher.appendTail(sb);
		html = sb.toString();
		return html;
	}
}
