package http;

import java.io.IOException;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import util.Constants;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;

public abstract class HttpServlet implements HttpHandler {
	private static Map<String, String> contentTypes = new HashMap<String, String>();
	static {
		contentTypes.put("txt", "text/plain");
		contentTypes.put("html", "text/html");
		contentTypes.put("htm", "text/html");
		contentTypes.put("gif", "image/gif");
		contentTypes.put("jpg", "image/jpg");
		contentTypes.put("png", "image/png");
		contentTypes.put("ico", "image/ico");
		contentTypes.put("js", "text/javascript");
		contentTypes.put("css", "text/css");
	}
	protected String getContentType(String url) {
		String[] elements = url.split("\\.");
		String extension = "txt";
		if (elements.length >= 2) {
			extension = elements[elements.length - 1];
		}
		return contentTypes.get(extension);
	}
	protected Map<String, String> getParameters(HttpExchange exchange) {
		Map<String, String> parameters = new HashMap<String, String>();
		URI url = exchange.getRequestURI();
		String query = url.getQuery();
		if (query.length() > 0) {
			String[] pairs = query.split("&");
			for (String pair : pairs) {
				String[] info = pair.split("=");
				parameters.put(info[0], info[1]);
			}
		}
		return parameters;
	}
	protected void log(String info) {
		if (Constants.DEBUG) {
			System.out.println(info);
		}
	}
	public void handle(HttpExchange exchange) throws IOException {
		log("[url] " + exchange.getRequestURI());
	}
}
